let authToken = '';
let ws = null;

$(document).ready(async function() {
  const urlParams = new URLSearchParams(window.location.search);
  authToken = urlParams.get('token');

  connectWS();
  connectMjpegWS();

  window.addEventListener('resize', (winSize) => {
    if (ws) {
      ws.send('{"command": "resend_images"}');
    }
  });
});

function connectWS() {
  let protocol = location.protocol !== 'https:' ? 'ws' : 'wss';
  ws = new WebSocket(protocol + '://' + window.location.host + '/local/storeoccupancymanager/ws', 'output');
  ws.binaryType = 'arraybuffer';

  ws.onopen = (event) => {
    console.log('WS - opened');
    // Clear old images
    let images = $('[id^=img_id_]');
    for (let i = 0; i < images.length; i++) {
      images[i].outerHTML = '';
    }
    // Authorize
    ws.send('{"authorization": "' + authToken + '"}');
  };

  ws.onclose = (event) => {
    console.log('WS - closed');
    setTimeout(connectWS, 5000);
  };

  ws.onmessage = (event) => {
    // Parse message
    let dataArr = new Uint8Array(event.data);
    let imageInfoSize = dataArr[3] | dataArr[2] << 8 | dataArr[1] << 16 | dataArr[0] << 24;
    let imageDataSize = dataArr[7] | dataArr[6] << 8 | dataArr[5] << 16 | dataArr[4] << 24;

    let imageInfo = String.fromCharCode.apply(null, new Uint8Array(event.data, 8, imageInfoSize));
    console.log('WS - ' + imageInfo);
    imageInfo = JSON.parse(imageInfo)

    if (imageDataSize) {
      // Show image
      let imgId = 'img_id_' + imageInfo.service_id;
      if ($('#' + imgId).length == 0) {
        let img = document.createElement("img");
        img.id = imgId;
        $('#img_container').append(img);
      }

      let imageData = Uint8ToString(new Uint8Array(event.data, 8 + imageInfoSize, imageDataSize));

      let ratio = Math.min($('#img_container').width() / imageInfo.output_width, $('#img_container').height() / imageInfo.output_height);
      let bgWidth = imageInfo.output_width * ratio;
      let bgHeight = imageInfo.output_height * ratio;
      let bgTop =  Math.floor(($('#img_container').height() - bgHeight) / 2);
      let bgLeft =  Math.floor(($('#img_container').width() - bgWidth) / 2);
      let top = bgTop + Math.floor((imageInfo.pos_y + 1.0) * 0.5 * bgHeight);
      let left = bgLeft + Math.floor((imageInfo.pos_x + 1.0) * 0.5 * bgWidth);

      let img = $('#' + imgId)[0];
      img.src = 'data:image/png;base64,' + btoa(imageData);
      img.style =
        'position: absolute;' +
        'top: ' + top + 'px;' +
        'left: ' + left + 'px;' +
        'width: ' + Math.floor(imageInfo.width * ratio) + 'px;' +
        'height: ' + Math.floor(imageInfo.height * ratio) + 'px;' +
        'z-index: 1000';
    } else {
      // Remove image
      let imgId = 'img_id_' + imageInfo.service_id;
      let img = $('#' + imgId);
      if (img.length != 0) {
        img[0].outerHTML = '';
      }
    }
  };
}

function connectMjpegWS() {
  let protocol = location.protocol !== 'https:' ? 'ws' : 'wss';
  const mjpegWs = new WebSocket(protocol + '://' + window.location.host + '/local/storeoccupancymanager/ws', 'mjpeg');
  mjpegWs.binaryType = 'arraybuffer';

  mjpegWs.onopen = (event) => {
    console.log('MJPEG WS - opened');
    mjpegWs.send('{"authorization": "' + authToken + '"}');
  };

  mjpegWs.onclose = (event) => {
    console.log('MJPEG WS - closed');
    setTimeout(connectMjpegWS, 5000);
  };

  mjpegWs.onmessage = (event) => {
    let dataArr = new Uint8Array(event.data);
    let imageInfoSize = dataArr[3] | dataArr[2] << 8 | dataArr[1] << 16 | dataArr[0] << 24;
    let imageDataSize = dataArr[7] | dataArr[6] << 8 | dataArr[5] << 16 | dataArr[4] << 24;

    let imageInfo = String.fromCharCode.apply(null, new Uint8Array(event.data, 8, imageInfoSize));
    imageInfo = JSON.parse(imageInfo)

    if (imageDataSize) {
      // Show image
      const imgId = 'img_background';
      if ($('#' + imgId).length == 0) {
        let img = document.createElement("img");
        img.id = imgId;
        $('#img_container').append(img);
      }

      let imageData = Uint8ToString(new Uint8Array(event.data, 8 + imageInfoSize, imageDataSize));

      let ratio = Math.min($('#img_container').width() / imageInfo.width, $('#img_container').height() / imageInfo.height);
      let width = imageInfo.width * ratio;
      let height = imageInfo.height * ratio;
      let top =  Math.floor(($('#img_container').height() - height) / 2);
      let left =  Math.floor(($('#img_container').width() - width) / 2);

      let img = $('#' + imgId)[0];
      img.src = 'data:image/jpeg;base64,' + btoa(imageData);
      img.style =
        'position: absolute;' +
        'top: ' + top + 'px;' +
        'left: ' + left + 'px;' +
        'width: ' + width + 'px;' +
        'height: ' + height + 'px;' +
        'z-index: 100';
    }
  }
}

function Uint8ToString(u8a){
  var CHUNK_SZ = 0x8000;
  var c = [];
  for (var i=0; i < u8a.length; i+=CHUNK_SZ) {
    c.push(String.fromCharCode.apply(null, u8a.subarray(i, i+CHUNK_SZ)));
  }
  return c.join("");
}